      Program BDSN2SMC

* Reformat BDSN data file into a SMC file.

* This is a special purpose program to reformat data at station PKD
* from the 2004 Parkfield mainshock provided by Chen Ji.
* Note that the number of samples differs for each trace.  My program is not 
* to take this into account automatically.  This will have to wait for a
* revision.

* Sample control file:
*! Control file for BDSN2SMC
*!Quake_Name
* Parkfield
*!Eq Hypocenter:
* 35.815, -120.374
*!Moment Magnitude:
* 6.0
*!sps
* 80
*!gain
* -5.056e+03
*!headers to skip
* 30
*!file (name followed by npts)
*14094992.BK.PKD.HLE.sac
* 20486
*14094992.BK.PKD.HLN.sac
* 20816
*14094992.BK.PKD.HLZ.sac
* 20716
*stop

* Dates: 09/30/04 - Written by D. Boore, patterned after cdmg2smc

      character f_ctl*40, f_sum*40, f_asc*40, f_smc*40, f_asc_4*4
      character thead_smc(11)*80, comments(100)*80, hdr_cmmnts(100)*79
      character headers_query*10, eqname*40
      integer ihead_smc(48)
      real rhead_smc(50)
      real rhead_in(70)
      integer ihead_in(40)

      real ts(:)
      allocatable :: ts
      

      logical f_ctl_exist

 
      int_null = -32768
      do i = 1, 48
        ihead_smc(i) = int_null
      end do
*      ihead_smc(13) = 0
*      ihead_smc(14) = 0

      real_null = 1.7e+38
      do i = 1, 50
        rhead_smc(i) = real_null
      end do

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = bdsn2smc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'bdsn2smc.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')
      write(nu_sum,'(a)') '  Summary program for program BDSN2SMC'      

      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
      eqname = ' '
      read(nu_ctl,'(a)') eqname
      call trim_c(eqname, nc_eqname)
 
      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
      read(nu_ctl,*) eqlat, eqlong
      
      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
      read(nu_ctl,*) eqdepth
      
      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
      read(nu_ctl,*) eqmag
      
      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
      read(nu_ctl,*) gain
      
      call skipcmnt(nu_ctl, hdr_cmmnts, n_hdr_cmmnts)
 
1234  continue

      f_asc = ' '
      read(nu_ctl,'(a)') f_asc
      call trim_c(f_asc,nc_f_asc)
      f_asc_4 = ' '
      f_asc_4 = f_asc(1:4)
      call upstr(f_asc_4)
      if(f_asc_4 .eq. 'STOP') go to 4321
      write(*,'(2a)') '  Processing file: ', f_asc(1:nc_f_asc)
      write(nu_sum,'(2a)') '  Processing file: ', f_asc(1:nc_f_asc)

      call get_lun(nu_asc)
      open(unit=nu_asc, file=f_asc(1:nc_f_asc), status='unknown')

      nrhead_in = 70
      read(nu_asc,*) (rhead_in(i), i = 1, nrhead_in)
      
      nihead_in = 40
      read(nu_asc,*) (ihead_in(i), i = 1, nihead_in)
      
      call skip(nu_asc, 8)
      
      read(nu_ctl, *) n_ts
      
* To be safe, add 10% more to the array:
      n_ts_max = 1.1*float(n_ts)

* Allocate array:
      allocate (ts(n_ts_max))

      read(nu_asc,*) (ts(i), i = 1, n_ts)
      
      
* Compute the mean:
      sum = 0.0
      do i = 1, n_ts
        sum = sum + ts(i)
      end do
      tsmean = sum/float(n_ts)
      call mnmaxidx(ts, 1, n_ts, 1,
     :             ymin, ymax, indx_min, indx_max) 
      write(nu_sum, '(a)') ' ymin, ymax, tsmean, gain = '
      write(nu_sum, *)  ymin, ymax, tsmean, gain

* Subtract the mean and scale the time series:
      do i = 1, n_ts
        ts(i) = (ts(i) - tsmean)/gain
      end do
       
      close(nu_asc)

* Get dt:

      dt = rhead_in(1)
      sps = 1/dt
 
      write(nu_sum,'(a,1x,i6,1x,f7.2)') '    n_ts, sps = ', n_ts, sps

      ihead_smc(17) = n_ts
      
      rhead_smc(2) = sps
      rhead_smc(3) = eqlat
      rhead_smc(4) = eqlong
      rhead_smc(5) = eqdepth
      rhead_smc(6) = eqmag
      stalat = rhead_in(32)
      stalong = rhead_in(33)
      staelev = rhead_in(33)
      
      rhead_smc(11) = stalat
      rhead_smc(12) = stalong
      rhead_smc(13) = staelev

      ihead_smc(17) = n_ts
      ihead_smc(32) = n_ts
      
* Get peak motions:
      
      call mnmaxidx(ts, 1, ihead_smc(17), 1,
     :             ymin, ymax, indx_min, indx_max) 
      tmin = float(indx_min-1)/rhead_smc(2)
      tmax = float(indx_max-1)/rhead_smc(2)
      rhead_smc(29) = tmax
      rhead_smc(30) = ymax
      rhead_smc(31) = tmin
      rhead_smc(32) = ymin

* Set up text headers:
      do i = 1, 11
        thead_smc(i) = '*'
      end do
      thead_smc(2) = '*'
      thead_smc(4)(27:80) = eqname(1:nc_eqname)
      thead_smc(5) = 
     :    'Moment Mag=          Ms=          Ml='
      write(thead_smc(5)(12:20),'(f9.2)') eqmag
      thead_smc(6) = 
     :    'station =                                component='
      thead_smc(7) = 
     :    'epicentral dist ='
* Extract station coordinates and compute epicentral distance:
      call distaz( -1.0, eqlat,eqlong,stalat,stalong,
     :    rdeg, rkm, az, baz)
      write(nu_sum,'(a)')
     :   ' eqlat, eqlong, stalat, stalong, rdeg, rkm = '
      write(nu_sum,*)
     :     eqlat, eqlong, stalat, stalong, rdeg, rkm
      write(thead_smc(7)(18:26),'(f9.1)') rkm
      thead_smc(8) = 
     :    'inst type=           data source = BDSN'
               
  
      ncomments = 0
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments) = '| File '//f_asc(1:nc_f_asc)//
     : ' reformatted by program BDSN2SMC'
      ncomments = ncomments + 1
      comments(ncomments) = 
     :          '| The following mean removed from the data '
      call trim_c(comments(ncomments), nc_c)
      write(comments(ncomments)(nc_c+1:nc_c+11),'(0p,1x,e10.3)')
     :  tsmean
      ncomments = ncomments + 1
      comments(ncomments) = '| Data values scaled by the factor '
      call trim_c(comments(ncomments), nc_c)
      write(comments(ncomments)(nc_c+1:nc_c+11),'(0p,1x,e10.3)')
     :  1.0/gain
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ihead_smc(16) = ncomments

      f_smc = ' '
      f_smc = f_asc(1:nc_f_asc)//'.smc'
      call trim_c(f_smc,nc_f_smc)

      write(*,'(a)') ' Writing file '//f_smc(1:nc_f_smc)
      write(nu_sum,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

      itype = 1
      call smcwrite(f_smc, ts, thead_smc, itype, 
     :              ihead_smc, rhead_smc, comments)
     
      deallocate (ts)


      go to 1234

4321  continue

      close(nu_ctl)
      close(nu_sum)

      stop
      end

      include '\forprogs\skip.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\distaz.for'
