
       Program CWB2SMC

*  Reads in ASCII files from Willie Lee from the
*  1999 Taiwan quake and reformats them
*  into SMC format (the format used on the CD-ROM)

*  For flexibility, information is read from a control file.
*  Here is an example:
* Control file for program CWB2SMC     ! first line of control file
*Name of summary file:
* smc_a900.sum
*Quake Params: date, time (iyr, imon, idom, ihr, imin)
* 1999 09 20 17 47
*Quake Params: location, magnitude (lat, long, h, Mw)
* 23.85  120.81  7  7.6 
*InstrType (A800, A900, IDSA, or ETNA) (4 characters, any column):
* A900
*Last part of filename for the three components:
* 1u.smc
* 2u.smc
* 3u.smc
*Pathname:
* r:\ascfiles\a900\
*Name of ascii file on CD
*PT001001.ASC
*PT003001.ASC
* stop

*  Dates:  09/24/99 - Started writing by D. Boore (based on
*                     ASCI2SMC.FOR)
*          09/24/99 - Reduce nsamps in output by 20 points to remove
*                     0.0 values.
*          02/15/00 - Redo for format from CD
*          06/30/00 - Added instrument type to control file, to make
*                     sure gain is proper
*          08/23/00 - Major changes, making it more general
*          08/25/00 - Modification to account for variable number of 
*                     blank lines between components (look for '$' in col. 1).
*                     Also increase dimensions from 20000 to 100000
*          09/02/00 - Change indx_comp so that N=1, V=2, E=3, and change read
*                     of time series to * to deal with different formats
*                     ETNA seems to be (5(i11,1x)); others (10(i6,1x))
*          09/13/00 - Get station name written in line 6 of the text headers
*                     from the Table 4 file rather than the ASCII file
*          01/18/01 - Renamed program TAIW2SMC to CWB2SMC and allowed for
*                     a new first line in the control file
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8
      
      real  rhead(50), sps, orient_h, orient_v, eqmag, 
     :      eqlat, eqlong, rnull

      integer ihead(48), iyr, imon, idom, idoy, ihr, isec,
     :        ncomments, inull

      integer idata(100000), nchar_f_out_end(3)
      real rdata(100000)

      character f_ctl*40, f_sum*40, f_out*40, f_in*40

      character chead(11)*80, comp_c(3)*1, staname_c(3)*6, 
     :          comments(10)*80, temp_sta_c*6, temp_comp_c*1, 
     :          type_c*40, f_tbl4*12, buf*80
      character f_out_end(3)*20, f_in_path*20

      character f_in_tbl4(300)*12, staname_tbl4(300)*6
      real dist_tbl4(300), lat_tbl4(300), long_tbl4(300), 
     :     elev_tbl4(300)      
      integer nchar_f_in_tbl4(300), nchar_staname_tbl4(300)
      logical f_ctl_exist

      inull = -32768
      rnull = +1.7e+38

* Open control file:

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') stop
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl, status='unknown')

      call skip(nu_ctl,2)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum, status='unknown')

      buf = ' '
      read(nu_ctl,'(a)') buf
      write(nu_sum,'(a)') buf
      read(nu_ctl,*) iyr, imon, idom, ihr, imin
      write(nu_sum,*) iyr, imon, idom, ihr, imin
      call DOY(iyr, imon, idom, idoy, istat)

      buf = ' '
      read(nu_ctl,'(a)') buf
      write(nu_sum,'(a)') buf
      read(nu_ctl,*) eqlat, eqlong, eqdepth, eqmag
      write(nu_sum,*) eqlat, eqlong, eqdepth, eqmag

      buf = ' '
      read(nu_ctl,'(a)') buf
      write(nu_sum,'(a)') buf
      type_c = ' '
      read(nu_ctl,'(a)') type_c
      write(nu_sum,'(a)') type_c
      call upstr(type_c)
      call trim_c(type_c, nch_type)
      if (type_c(1:4) .eq. 'A800') then
        scale_factr = 980.0*1.0/2048.0
        f_tbl4 = 'tabl4pt1.txt'
      else if (type_c(1:4) .eq. 'A900') then
        scale_factr = 980.0*2.0/32768.0
        f_tbl4 = 'tabl4pt2.txt'
      else if (type_c(1:4) .eq. 'IDSA') then
        scale_factr = 980.0*2.0/32768.0
        f_tbl4 = 'tabl4pt3.txt'
      else if (type_c(1:4) .eq. 'ETNA') then
        scale_factr = 980.0*2.0/8388608.0
        f_tbl4 = 'tabl4pt4.txt'
      else
        write(*,'(3a)') ' type_c = ', type_c(1:4), 
     :       ' NOT RECOGNIZED: QUITTING!!!'      
        stop
      end if

      write(nu_sum,'(2a)') ' f_tbl4 = ', f_tbl4

      buf = ' '
      read(nu_ctl,'(a)') buf
      write(nu_sum,'(a)') buf
      do i = 1, 3
        f_out_end(i) = ' '
        read(nu_ctl,'(a)') f_out_end(i)
        write(nu_sum,'(a)') f_out_end(i)
        call upstr(f_out_end(i))
        call trim_c(f_out_end(i),nchar_f_out_end(i))
      end do

      buf = ' '
      read(nu_ctl,'(a)') buf
      write(nu_sum,'(a)') buf
      f_in_path = ' '
      read(nu_ctl,'(a)') f_in_path
      write(nu_sum,'(a)') f_in_path
      call upstr(f_in_path)
      call trim_c(f_in_path,nchar_f_in_path)

* Extract info from table containing lat and long:
      call get_lun(nu_tbl4)        
      open(unit=nu_tbl4, file=f_tbl4, status='unknown')
      call skip(nu_tbl4,1)
      do i = 1, 300
        f_in_tbl4(i) = ' '
        staname_tbl4(i) = ' '
      end do 
      ntbl4 = 0
1111  continue
        buf = ' '
        read(nu_tbl4,'(a)', end=9999) buf
        ntbl4 = ntbl4 + 1
        call CSRC(1,buf,f_in_tbl4(ntbl4),nchar_f_in_tbl4(ntbl4))
        call upstr(f_in_tbl4(ntbl4))
        call CSRC(2,buf,staname_tbl4(ntbl4),nchar_staname_tbl4(ntbl4))
        call upstr(staname_tbl4(ntbl4))
        call CSRF(3,buf,dist_tbl4(ntbl4),ISTAT)
        call CSRF(7,buf,lat_tbl4(ntbl4),ISTAT)
        call CSRF(8,buf,long_tbl4(ntbl4),ISTAT)
        call CSRI(9,buf,ielev,ISTAT)
        elev_tbl4(ntbl4) = ielev
      goto 1111
9999  continue
      close(nu_tbl4)

* DEBUG
*      call get_lun(nu_debug)
*      open(unit=nu_debug, file='debug.out', status='unknown')
*      do i = 1, ntbl4
*        write(nu_debug,'(a,1x,a,2(1x,f8.3))') 
*     :   f_in_tbl4(i), staname_tbl4(i), lat_tbl4(i), long_tbl4(i)
*      end do
* DEBUG

      buf = ' '
      read(nu_ctl,'(a)') buf
      write(nu_sum,'(a)') buf

2222  continue   ! Big loop to read individual files
*      write(nu_sum,'(a)') '  AFTER 2222 CONTINUE'
      
      f_in = ' '
      read(nu_ctl,'(a)') f_in
      call upstr(f_in)
      call trim_c(f_in,nchar_f_in)
      if(f_in(1:nchar_f_in) .eq. 'STOP') then
        close (unit=nu_ctl)
        close (unit=nu_sum)
        stop
      end if
      write(nu_sum,'(a)') f_in(1:nchar_f_in)

* DEBUG
*      do i = 1, ntbl4
*        write(nu_debug,'(1x,i2,1x,i2,a,1x,a,2(1x,f8.3))') 
*     :   nchar_f_in, nchar_f_in_tbl4(i), 
*     :   f_in_tbl4(i), staname_tbl4(i), lat_tbl4(i), long_tbl4(i)
*      end do
* DEBUG

* Find index of tabl4 for this file:
      do i = 1, ntbl4
        if(f_in(1:nchar_f_in) .eq. 
     :     f_in_tbl4(i)(1:nchar_f_in_tbl4(i))) then
          jtbl4 = i
          go to 1234
        end if
      end do
      write(*, '(5a)') ' ERROR: could not find file ', 
     :  f_in(1:nchar_f_in), ' in file ', 
     :  f_tbl4, ', QUITTING!!'
      write(nu_sum, '(5a)') ' ERROR: could not find file ', 
     :  f_in(1:nchar_f_in), ' in file ', 
     :  f_tbl4, ', QUITTING!!'
      close(nu_ctl)
      close(nu_sum)
      stop
1234  continue

      stalat = lat_tbl4(jtbl4)
      stalong = long_tbl4(jtbl4)
      
      call distaz( +1.0, eqlat, eqlong, stalat, stalong,
     * rdeg, rkm, az, baz)
 
      f_in = f_in_path(1:nchar_f_in_path)//f_in(1:nchar_f_in)
      nchar_f_in = nchar_f_in_path + nchar_f_in
      
      write(*,'(2a)') ' f_in = ', f_in
*      write(nu_sum,'(2a)') ' f_in = ', f_in

      write(nu_sum,'(2a)') ' For f_in = ', f_in
      write(nu_sum,'(a,1x,i4,a,2(1x,f8.3))') 
     :   '  jtbl4 = ', jtbl4, 
     :   ' and stalat, stalong =', stalat, stalong

      call get_lun(nu_in)
      open(unit=nu_in, file=f_in, status='unknown')

* Skip initial headers:
      call skip(nu_in,6)   ! used to be '5'

* Big Loop over components to obtain data:
      do i_comp = 1, 3

* Position to line after '$':
888     continue
        buf = ' '
        read(nu_in,'(a)') buf
        if (buf(1:1) .eq. '$') goto 777
        goto 888
777     continue
        call skip(nu_in,1)

        temp_sta_c = ' '
        read(nu_in,'(a)') temp_sta_c
        if (temp_sta_c .eq. ' ') then
          temp_sta_c = 'unk'
        end if
        call upstr(temp_sta_c)
        call trim_c(temp_sta_c, nchar_temp_sta_c)
*        write(nu_sum,'(2a)')
*     :   ' temp_sta_c = ', temp_sta_c(1:nchar_temp_sta_c)
        temp_comp_c = ' '
        read(nu_in,'(a)') temp_comp_c
        call upstr(temp_comp_c)

* Commented out next portion of code because the station name
* in the file is not usually the same as in the table; the table
* name is more complete (e.g., TCU129 rather than T129)
*        write(nu_sum,'(a)') ' BEFORE IF(TEMP_STA...)'
*        if(temp_sta_c(1:nchar_temp_sta_c) .ne. 
*     :     staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4))) then
*          write(*, '(3a)') ' ERROR: station name from ascii file (',
*     :      temp_sta_c(1:nchar_temp_sta_c), ')'
*          write(*, '(3a)') ' not equal to the name from table 4 (',
*     :      staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4)), 
*     :      '); QUITTING!!'
*          write(nu_sum, '(3a)') 
*     :      ' ERROR: station name from ascii file (',
*     :      temp_sta_c(1:nchar_temp_sta_c), ')'
*          write(nu_sum, '(3a)') ' not equal to the name from table 4 (',
*     :      staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4)), 
*     :      '); QUITTING!!'
*          close(nu_ctl)
*          close(nu_sum)
*          stop
*        end if            
*        write(nu_sum,'(a)') ' AFTER IF(TEMP_STA...)'

        if(temp_comp_c .eq. 'V') then
          orient_v = 0.0
          orient_h = 0.0
          indx_comp = 2
        else if(temp_comp_c .eq. 'N') then
          orient_v = 90.0
          orient_h = 0.0
          indx_comp = 1
        else if(temp_comp_c .eq. 'E') then
          orient_v = 90.0
          orient_h = 90.0
          indx_comp = 3
        else
          write(nu_sum,'(3a)') 
     :      ' temp_comp_c = ',
     :      ' temp_comp_c = ',
     :      '; .ne. V, N, or E: QUITTING!!!'
          stop
*          orient_v = rnull
*          orient_h = rnull
        end if

        call skip(nu_in, 7)
        read(nu_in,*) nsamps
        read(nu_in,*) sps
        call skip(nu_in, 6)
        read(nu_in, * ) (idata(i), i = 1, nsamps)            
*        read(nu_in,'(10(i6,1x))') (idata(i), i = 1, nsamps)            
* scale data:
        do i = 1, nsamps
          rdata(i) = scale_factr * idata(i)
        end do
* finished reading this component

* Open output file:
        f_out = ' '
        f_out = staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4))//
     :          f_out_end(indx_comp)(1:nchar_f_out_end(indx_comp))
*        write(nu_sum,'(a,1x,i4)') ' indx_comp = ', indx_comp
*        write(nu_sum,'(2a)') 
*     :    ' staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4)) = ',
*     :      staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4))
*        write(nu_sum,'(a,1x,i4)') 
*     :    ' nchar_f_out_end(indx_comp) = ', 
*     :      nchar_f_out_end(indx_comp)
*        write(nu_sum,'(2a)')
*     :    ' f_out_end(indx_comp)(1:nchar_f_out_end(indx_comp)) = ',
*     :      f_out_end(indx_comp)(1:nchar_f_out_end(indx_comp))
        write(nu_sum,'(2a)') ' Output file = ', f_out

* Write name to screen:
        write(*, '(2a)') ' Output file name: ', f_out

* Write headers:

        chead(1) = '*'
        chead(2) = '*'

        write(chead(3),'(a)') 
     :    staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4))

        write(chead(4),'(5x,i4,2x,i2,2x,i2,4x,i2,i2,1x,a)') 
     :              iyr, imon, idom, ihr, imin,'Chi-Chi (Taiwan)'
        chead(5) = 
     :       'Moment Mag=          Ms=          Ml='
        write(chead(5)(12:20),'(f9.2)') eqmag
        write(chead(5),'(a,6x,f3.1)') 'Moment Mag=', eqmag
        chead(6) = 
     :    'station =                                component='
        chead(6)(11:nchar_staname_tbl4(jtbl4)+10) = 
     :     staname_tbl4(jtbl4)(1:nchar_staname_tbl4(jtbl4))
        if (orient_v .eq. 0.0) then      ! assume vertical
          chead(6)(56:57) = 'UP'
        else
          write(chead(6)(53:57),'(i5)') int(orient_h)
        end if
        write(chead(7),'(a,t34,a)') 'epicentral dist =','pk acc ='
        write(chead(7)(18:26),'(f9.1)') rkm
        write(chead(8),'(a, t22, a)') 
     :          'inst type=DSA', 'data source = CWB'
        chead(9) = '*'
        chead(10) = '*'
        chead(11) = '*'

* Set up comments:
        comments(1) = '| CWB file '//f_in
        comments(2) = '| Reformatted to SMC format, using TAIW2SMC'
        ncomments = 2
             
* Now fill the integer header block:
* First fill with null values:
        do i = 1, 48
          ihead(i) = inull
        end do

        ihead(2) = iyr
        ihead(3) = idoy
        ihead(4) = ihr
        ihead(5) = imin
        ihead(9) = indx_comp
        ihead(13) = orient_v
        ihead(14) = orient_h
        ihead(16) = ncomments
        ihead(17) = nsamps - 20
 
      precision = ' '
      precision = 'STANDARD'
      ihead_smc(47) = ihead_smc(1)

* Now fill the real header block:
* First fill with null values:
        do i = 1, 50
          rhead(i) = rnull
        end do
   
        rhead(2) = sps
        rhead(3) = eqlat
        rhead(4) = eqlong
        rhead(5) = eqdepth
        rhead(6) = eqmag
        rhead(11) = stalat
        rhead(12) = stalong
        rhead(13) = elev_tbl4(jtbl4)
        rhead(17) = rkm
        rhead(18) = az
  
* Get peak motions:
      
        call mnmaxidx(rdata, 1, ihead(17), 1,
     :             ymin, ymax, indx_min, indx_max) 
        tmin = float(indx_min-1)/sps
        tmax = float(indx_max-1)/sps
        rhead(29) = tmax
        rhead(30) = ymax
        rhead(31) = tmin
        rhead(32) = ymin

         
        itype = 1
        call SMCWrite(f_out, precision,
     :                rdata, chead, itype, 
     :                 ihead, rhead, comments)

* That should be it; loop back for another component

      end do  ! end of loop over i_comp

      close(nu_in)

      go to 2222

      end

      include '\forprogs\skip.for'
      include '\forprogs\doy.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\distaz.for'
      include '\forprogs\csrc.for'
      include '\forprogs\csrf.for'
      include '\forprogs\csri.for'
      include '\forprogs\csr123.for'



