      Program PEA2SMC

* Reformat into a smc file an ascii file made by PEA.

* Sample control file:
* first line below starts the control, with line 1, col 1 = "N" in "Nominal":
*! Control file for PEA2SMC      ! first line
*! Revision of program involving a change in the control file on this date:
*   02/08/12
*! Name of summary file:
* pea2smc.sum
*! Xfactr
* 981.0
*!Nskip
* 3
*!Format for line containing npts and dt 
*! ('*' for free format; '(t6,i7,t18,f8.5)' for new format)
* *  
*! (t6,i7,t18,f8.5)
*!Format for the data
* (5f15.7)   ! The standard PEA format
*! (5f14.6)   ! 
*!Input file name ("stop" in any column to quit):
* \smsim\ts0.col
* \smsim\ts1.col
*STOP

* Dates: 05/13/04 - Written by D. Boore, patterned after asc2smc
*        12/19/04 - output file made by adding "smc" to end of input file name.
*        06/16/05 - Replace formatted read of n_ts, dt with unformatted read
*        06/26/05 - Writes modified smc format (6 significant digits, consistent
*                   with the PEA files).
*        09/23/05 - Remove the modification to write 6 significant digits
*        01/11/07 - Include text headers as comments
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        07/14/08 - Force use of higher precision smc output
*        10/06/09 - Increase the length of the file names
*        10/07/09 - Put read code into a subroutine (pea_read.for)
!        11/19/10 - Use date in control file
!                 - Use skipcmnt to read comment lines
!                 - Read format to be used in reading npts and dt.
!        10/05/11 - Modernize code somewhat, use allocatable arrays
!                 - Move pea_read code into this main program (because 
!                   of using allocatable arrays).
!        02/08/12 - Read in format of the data
!        12/08/13 - Check for existence of the input file
!        10/10/20 - Use pea_get_npts_sps.for to get npts and allocate the ts array before calling
!                   read_pea to read the file.

      character precision*8
      

      real ts(:)
      allocatable :: ts

      character f_ctl*400, f_sum*400, f_asc*400, f_smc*400, f_asc_4*4
      character char_head(11)*80, comments(100)*80,  
     :                            ctl_cmmnts(100)*79,
     :                            date_ctl_correct*8, date_ctl_in*30
 
      character headers_query*10, format4npts_dt*20, format4data*60
      integer int_head(48)
      real real_head(50)

      character hdr_cmmnts(10)*200
      real xfactr, dt, sps
      integer nc_f_asc, nu_asc, nskip, n_ts, ncomments
      integer nc_hdr_cmmnts(10)

      logical f_ctl_exist, f_exist

      do i = 2, 11
        char_head(i) = '*'
      end do

      int_null = -32768
      do i = 1, 48
        int_head(i) = int_null
      end do
!      int_head(13) = 0
!      int_head(14) = 0

      real_null = 1.7e+38
      do i = 1, 50
        real_head(i) = real_null
      end do

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = pea2smc.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'pea2smc.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/08/12'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')
      write(nu_sum,'(a)') '  Summary program for program PEA2SMC'      

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl,*) xfactr

      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl,*) nskip
      
      format4npts_dt = ' '
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl,'(a)') format4npts_dt
      call trim_c(format4npts_dt, nc_format4npts_dt)
      
      format4data = ' '
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)
      read(nu_ctl,'(a)') format4data
      call trim_c(format4data, nc_format4data)
      index_right_paren = index(format4data(1:nc_format4data),')')
      if (index_right_paren == 0) then
        print *,' ERROR: right parenthesis not found in '//
     :    format4data(1:nc_format4data)//'; QUIT'
        stop
      end if
      
      
      call skipcmnt(nu_ctl,ctl_cmmnts, nc_ctl_cmmnts)

      loop over files: DO

        f_asc = ' '
        read(nu_ctl,'(a)') f_asc
        call trim_c(f_asc,nc_f_asc)
        f_asc_4 = ' '
        f_asc_4 = f_asc(1:4)
        call upstr(f_asc_4)
        if(f_asc_4 .eq. 'STOP') EXIT

        f_exist = .false.
        inquire(file=f_asc(1:nc_f_asc), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE '//f_asc(1:nc_f_asc)//
     :          ' DOES NOT EXIST, CYCLE To NEXT FILE ******* '
          CYCLE loop over files
        end if
 

        write(nu_sum,'(2a)') '  Processing file: ', f_asc(1:nc_f_asc)

        call pea_get_npts_sps(f_asc, nc_f_asc,
     :              nskip, 
     :              format4npts_dt, nc_format4npts_dt,
     :              n_ts, dt, sps)

        allocate(ts(n_ts))


        call pea_read(f_asc, nc_f_asc, nskip, 
     :                format4npts_dt, nc_format4npts_dt,
     :                comments, ncomments, xfactr,
     :                n_ts, dt, sps, ts)
      
!        call get_lun(nu_asc)
!       
!        open(unit=nu_asc, file=f_asc(1:nc_f_asc), status='unknown')
!
!        do i = 1, nskip
!          hdr_cmmnts(i) = ' '
!          read(nu_asc,'(a)') hdr_cmmnts(i)
!          call trim_c(hdr_cmmnts(i), nc_hdr_cmmnts(i))
!        print *,' hdr_cmmnts:'//hdr_cmmnts(i)(1:nc_hdr_cmmnts(i))
!        end do
!      
!        ncomments = 0
!        ncomments = ncomments + 1
!        comments(ncomments) = '|'
!        do i = 1, nskip
!          ncomments = ncomments + 1
!          comments(ncomments) = '|'//hdr_cmmnts(i)(1:nc_hdr_cmmnts(i))
!        end do
!
!        if (format4npts_dt(1:nc_format4npts_dt) == '*') then
!          read(nu_asc,*) n_ts, dt
!        else
!          read(nu_asc,format4npts_dt(1:nc_format4npts_dt)) n_ts, dt
!        end if
! 
!        allocate(ts(n_ts))
!
!
!        read(nu_asc,format4data(1:index_right_paren)) (ts(i),i=1,n_ts)
!        read(nu_asc,'(5e14.6)') (ts(i),i=1,n_ts)
!        read(nu_asc,'(5e15.7)') (ts(i),i=1,n_ts)
!      
!        close(nu_asc)
!
!
! Scale the time series:
!        do i = 1, n_ts
!          ts(i) = xfactr * ts(i)
!        end do
!
!        sps = 1.0/dt
      
        int_head(17) = n_ts
      
        real_head(2) = sps

        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = '| File '//f_asc(1:nc_f_asc)//
     :   ' reformatted by program PEA2SMC'
        ncomments = ncomments + 1
        comments(ncomments) = '| Data values scaled by the factor '
        call trim_c(comments(ncomments), nc_c)
        write(comments(ncomments)(nc_c+1:nc_c+11),'(0p,1x,e10.3)')
     :    xfactr
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments

        f_smc = ' '
!      f_smc = f_asc(1:nc_f_asc)//'.smc6'
        f_smc = f_asc(1:nc_f_asc)//'.smc'
        call trim_c(f_smc,nc_f_smc)

        write(*,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

        precision = ' '
!      precision = 'STANDARD'
        precision = 'HIGH'
        int_head(47) = int_head(1)

        write(nu_sum,'(a)') ' Writing file '//f_smc(1:nc_f_smc)

        call smcwrite(f_smc, precision,
     :                ts, char_head, 0, 
     :                int_head, real_head, comments)
     
        deallocate(ts)

      END DO loop over files

 
      close(nu_ctl)
      close(nu_sum)

      stop
      end

      include '\forprogs\distaz.for'
      include '\forprogs\skip.for'
      include '\forprogs\smcwrite6.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\pea_get_npts_sps.for'
      include '\forprogs\pea_read.for'
      
      
