* ---------------------------------------------------------------- USCV3GS
      Program USCV3GS

* Convert USC V3 files to USGS SMC format

c       per(i) : period - sec 
c       fas(i) : Fourier amplit. of acceleration  - in/sec
c       rd(i) : spectral displacement            - in
c       rv(i) : spectral velocity                - in/sec
c       aa(i) : spectral acceleration            - g
c       pssv(i) : pseudo velocity spectrum                 - in/sec
c       tprd(i) : the time of peak displacement responses  - sec
c       tprv(i) : times of peak velocity responses         - sec
c       tpaa(i) : times of peak acceleration responses     - sec

* Units converted to cm, cm/s, cm/s/s

* Sample control file:
*Control file for program USCV3GS
*Eq Hypocenter:
* 34.594, -116.271
*Moment Magnitude:
* 7.1
*Station Filenames:
*v2x0100.dat
*v2x0401.dat
* STOP

* Dates: 05/24/01 - Written by D. Boore, patterned after CDMGV2GS

      character thead_smc(11)*80, comments(200)*80
      integer ihead_smc(48)
      real rhead_smc(50)

      real damp(5), per(100), fas(100)
      real rd(100,5), pssv(100,5), rv(100,5), aa(100,5)
      real tprd(100,5),tprv(100,5),tpaa(100,5)

      character thead_usc(30)*80, c_lat*1, c_long*1, c_ncomp*1
      integer ihead_usc(100)
      real rhead_usc(100)

      character f_ctl*40, f_in*40, f_out*40, endchn*2, c_stop*4
      character temp_c*8, temp_staid*5, buf*80, avd_c*3
      logical f_ctl_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') stop
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skip(nu_ctl,2)
      read(nu_ctl, *) eqlat, eqlong
      call skip(nu_ctl,1)
      read(nu_ctl, *) amommag

      call skip(nu_ctl,1)
     
1234  continue

      f_in = ' '
      read(nu_ctl,'(a)') f_in
      call trim_c(f_in, nc_f_in)
      c_stop = ' '
      c_stop = f_in(1:4)
      call upstr(c_stop)
      if(c_stop.eq.'STOP') then
        close(nu_ctl)
        stop
      end if

      write(*,'(a)') '  Processing file '//f_in(1:nc_f_in)

      call get_lun(nu_in)
      open(unit=nu_in,file=f_in,status='unknown')

* Loop over channels:
  
      nchn = 0

2000  continue

* Read text headers:
      nhead_usc = 30
      do i = 1, nhead_usc
        thead_usc(i) = ' '
        read(nu_in,'(a)', end=666) thead_usc(i)
      end do

      nchn = nchn + 1

* Check for more than 3 channels:
      if (nchn .gt. 3) then
         write(*,'(a,1x,i5,a)') 
     :       ' !!! nchn = ', nchn, ', > 3; QUITTING !!!'
      end if

      ndimen = 100

* Read integer headers:
      read(nu_in,'(16i5)') (ihead_usc(i), i=1,ndimen)

* Read real headers:
      read(nu_in, '(8f10.3)') (rhead_usc(i), i=1,ndimen)

      ndamp = 5
      read(nu_in,'(8f10.3)') (damp(i), i = 1, ndamp)
      read(nu_in,'(8f10.3)') (per(i), i = 1, ndimen)
c       Read the Fourier amplitude spectrum:
      read(nu_in,*) ! skip header

      nrs = ihead_usc(68)

* Read FAS:
      read(nu_in,'(8e10.3)') (fas(i), i = 1, ndimen)
* Change units to cm/s
      do i = 1, ndimen
        fas(i) = 2.54 * fas(i)  ! in/sec to cm/sec
      end do

c  Read the response spectrum:
*  "do" loop for the different values for the damping:
      do j = 1, 5
        read(nu_in,*)  ! skip header
        read(nu_in,'(8e10.3)') (rd(i,j), i = 1, ndimen)
        read(nu_in,'(8e10.3)') (rv(i,j), i = 1, ndimen)
        read(nu_in,'(8e10.3)') (aa(i,j), i = 1, ndimen)
        read(nu_in,'(8e10.3)') (pssv(i,j), i = 1, ndimen)
        read(nu_in,'(8e10.3)') (tprd(i,j), i = 1, ndimen)
        read(nu_in,'(8e10.3)') (tprv(i,j), i = 1, ndimen)
        read(nu_in,'(8e10.3)') (tpaa(i,j), i = 1, ndimen)
* Change units to cm, cm/s, cm/s/s
        do i = 1, ndimen
          rd(i,j) = 2.54 * rd(i,j)
          rv(i,j) = 2.54 * rv(i,j)
          pssv(i,j) = 2.54 * pssv(i,j)
          aa(i,j) = 981.0 * aa(i,j) ! g to cm/s/s         
        end do
      end do

* Set up text headers:
      do i = 1, 11
        thead_smc(i) = ' '
      end do
      thead_smc(2) = '*'
      thead_smc(3) = thead_usc(7)(13:17) ! stanum (Not official SMC format)
      thead_smc(4)(1:25) = thead_usc(5)(1:25) !eqdate,time (Not ofcl SMC format)
      thead_smc(4)(27:66) = thead_usc(4)(1:40) !eqname (Not official SMC format)
      thead_smc(5) = 
     :    'Moment Mag=          Ms=          Ml='
      write(thead_smc(5)(12:20),'(f9.2)') amommag
      thead_smc(6) = 
     :    'station =                                component='
      thead_smc(6)(11:40) = thead_usc(8)(1:30) ! staname

      if (ihead_usc(27) .gt. 360) then      ! assume vertical
        thead_smc(6)(56:57) = 'UP'
      else
        write(thead_smc(6)(53:57),'(i5)') ihead_usc(27)
      end if
      thead_smc(7) = 
     :    'epicentral dist ='
* Extract station coordinates and compute hypocentral distance:
*      stalat = real_head(9)
*      stalong = real_head(10)

* A more accurate station location is given using d, m, s, assuming that
* s is given accurately.

      alat_d = ihead_usc(10)
      alat_m = ihead_usc(11)
      alat_s = ihead_usc(12)
      stalat = alat_d + (alat_m + alat_s/60.0)/60.0
      along_d = ihead_usc(13)
      along_m = ihead_usc(14)
      along_s = ihead_usc(15)
      stalong = along_d + (along_m + along_s/60.0)/60.0

      if (stalong .gt. 0.0) stalong = -stalong   ! correct sign of longitude

* Here is more general code:
*      read(thead_usc(6)(23:28),'(f6.3)') stalat
*      read(thead_usc(6)(34:40),'(f7.3)') stalong
*      c_lat = thead_usc(6)(30:30)
*      c_long = thead_usc(6)(42:42)
*      call upstr(c_lat)
*      if (c_lat .eq. 'S') stalat = -abs(stalat)
*      call upstr(c_long)
*      if (c_long .eq. 'W') stalong = -abs(stalong)

      call distaz( -1.0, eqlat,eqlong,stalat,stalong,
     :    rdeg, rkm, az, baz)
      write(thead_smc(7)(18:26),'(f9.1)') rkm
      thead_smc(8) = 
     :    'inst type=           data source = USC'
               
*      temp_c = ' '
*      temp_c = thead_usc(6)(40:47) 
*      call trim_c(temp_c,nc_tc)
*      if (nc_tc .gt. 5) nc_tc = 5
*      thead_smc(8)(11:11+nc_tc-1) = temp_c(1:nc_tc)

      do i = 9, 11
        thead_smc(i) = '*'
      end do

* Set up comments:
      comments(1) = '|'
      comments(2) = '| USC file '//f_in
      comments(3) = '| Reformatted to SMC format, using USCV3GS'
      comments(4) = '|'
      comments(5) = '| Here follow the USC text headers:'
      comments(6) = '|'
      ncomments = 6
* Now add USC text headers as comments:
      do i = 1, nhead_usc
        comments(i+ncomments) = '| '//thead_usc(i)(1:78)
      end do
      ncomments = ncomments + nhead_usc

* Set up integer headers independent of type of motion:
      inull = -32768
      do i = 1, 48
        ihead_smc(i) = inull
      end do

      read(thead_usc(6)(54:57),'(i4)') ihead_smc(8)

      ihead_smc(9) = nchn

      if (ihead_usc(27) .gt. 360) then      ! assume vertical
        ihead_smc(13) = 0
        ihead_smc(14) = 0
      else
        ihead_smc(13) = 90
        ihead_smc(14) = ihead_usc(27)
      end if

      ihead_smc(16) = ncomments

      temp_staid = ' '
      temp_staid = thead_usc(6)(13:17)
      call notnumrc(temp_staid, 5, iflag)
      if (iflag .eq. 0) then
        read(temp_staid,'(i5)') ihead_smc(30)
      end if

      ihead_smc(31) = 1

* Set up real headers independent of type of motion:
      rnull = 1.7e+38
      do i = 1, 50
        rhead_smc(i) = rnull
      end do

      rhead_smc(2) = 1.0/rhead_usc(61) ! this assumes that dt is same for a,v,d
      rhead_smc(3) = eqlat
      rhead_smc(4) = eqlong
      rhead_smc(6) = amommag
      rhead_smc(11) = stalat
      rhead_smc(12) = stalong
      rhead_smc(17) = rkm
      rhead_smc(18) = az
      rhead_smc(22) = 1.0/rhead_usc(1)  ! natural frequency
      rhead_smc(23) = rhead_usc(2)      ! damping
      rhead_smc(24) = rhead_usc(6)      ! sensitivity
      rhead_smc(33) = rhead_usc(62) - rhead_usc(63) ! lc start
      rhead_smc(34) = rhead_usc(62)                 ! lc stop
      rhead_smc(35) = rhead_usc(58) - rhead_usc(59) ! hc start
      rhead_smc(36) = rhead_usc(58)                 ! hc stop

      f_out = ' '
      nstem = index(f_in(1:nc_f_in),'.') - 1
      write(c_ncomp,'(i1)') nchn
      f_out = f_in(1:nstem)//'.rs'//c_ncomp
      call trim_c(f_out, nc_f_out)
      itflag = 1
      write(*,'(a)') '  Writing output file '//f_out(1:nc_f_out)
      call rs2write(f_out, 
     :        damp, ndamp, per, nrs, rd, rv, aa, 
     :        thead_smc, ihead_smc, rhead_smc, comments, 
     :        itflag, tprd, tprv, tpaa)

*  check for end-of-channel marker:
      buf = ' '
      read(nu_in, '(a)') buf
      endchn = ' '
      call RCC (1,buf,80,endchn,NC,ibgn,iend)

      if (endchn(1:2) .ne. '/&') then
         write(*,'(a)') ' Problem with endchn, ne /&, quitting'
         stop
      end if

*  go back for a new channel:

      go to 2000

666   continue
      close(nu_in)

      go to 1234  ! return for another file

      end
* ---------------------------------------------------------------- USCV3GS

      include '\forprogs\notnumrc.for'
      include '\forprogs\rs2write.for'
      include '\forprogs\upstr.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\distaz.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\rc_subs.for'

     